@@ -16,7 +16,7 @@ var (
 )
 
 type TestClient struct {
-	Data prefixfile.VRPList
+	Data prefixfile.RPKIList
 
 	InitSerial bool
 	Serial     uint32
@@ -34,9 +34,9 @@ func getBasicClientConguration(version int) ClientConfiguration {
 
 func getClient() *TestClient {
 	return &TestClient{
-		Data: prefixfile.VRPList{
+		Data: prefixfile.RPKIList{
 			Metadata: prefixfile.MetaData{},
-			Data:     make([]prefixfile.VRPJson, 0),
+			ROA:      make([]prefixfile.VRPJson, 0),
 		},
 		InitSerial: InitSerial,
 		Serial:     Serial,
@@ -126,30 +126,3 @@ func TestRouterKeyEncodeDecode(t *testing.T) {
 		t.FailNow()
 	}
 }
-
-func TestASPAEncodeDecode(t *testing.T) {
-	p := &PDUASPA{
-		Version:           1,
-		Flags:             1,
-		AFIFlags:          1,
-		ProviderASCount:   2,
-		CustomerASNumber:  64497,
-		ProviderASNumbers: []uint32{64498, 64499},
-	}
-
-	buf := bytes.NewBuffer(nil)
-	p.Write(buf)
-
-	outputPdu, err := Decode(buf)
-
-	if err != nil {
-		t.FailNow()
-	}
-
-	orig := fmt.Sprintf("%#v", p)
-	decode := fmt.Sprintf("%#v", outputPdu)
-	if orig != decode {
-		t.Fatalf("%s\n is not\n%s", orig, decode)
-		t.FailNow()
-	}
-}
