@@ -3,22 +3,19 @@ package rtrlib
 import (
 	"bytes"
 	"crypto/tls"
-	"flag"
 	"fmt"
 	"io"
+	"math"
 	"math/rand"
 	"net"
+	"net/netip"
 	"sync"
-	"time"
 
 	"golang.org/x/crypto/ssh"
 )
 
 func GenerateSessionId() uint16 {
-	var sessid uint16
-	r := rand.New(rand.NewSource(time.Now().UTC().Unix()))
-	sessid = uint16(r.Uint32())
-	return sessid
+	return uint16(rand.Intn(math.MaxUint16 + 1))
 }
 
 type RTRServerEventHandler interface {
@@ -32,7 +29,7 @@ type RTREventHandler interface {
 	RequestNewVersion(*Client, uint16, uint32)
 }
 
-// This is a general interface for things like a VRP, BGPsec Router key or ASPA object
+// This is a general interface for things like a VRP or BGPsec Router key
 // Be sure to have all of these as pointers, or SetFlag() cannot work!
 type SendableData interface {
 	Copy() SendableData
@@ -44,10 +41,10 @@ type SendableData interface {
 	GetFlag() uint8
 }
 
-// This handles things like ROAs, BGPsec Router keys, ASPA info etc
+// This handles things like ROAs, BGPsec Router keys info etc
 type SendableDataManager interface {
-	GetCurrentSerial(uint16) (uint32, bool)
-	GetSessionId() uint16
+	GetCurrentSerial() (uint32, bool)
+	GetSessionId(uint8) uint16
 	GetCurrentSDs() ([]SendableData, bool)
 	GetSDsSerialDiff(uint32) ([]SendableData, bool)
 }
@@ -65,24 +62,24 @@ func (e *DefaultRTREventHandler) RequestCache(c *Client) {
 	if e.Log != nil {
 		e.Log.Debugf("%v > Request Cache", c)
 	}
-	sessionId := e.sdManager.GetSessionId()
-	serial, valid := e.sdManager.GetCurrentSerial(sessionId)
+	sessionId := e.sdManager.GetSessionId(c.GetVersion())
+	serial, valid := e.sdManager.GetCurrentSerial()
 	if !valid {
 		c.SendNoDataError()
 		if e.Log != nil {
 			e.Log.Debugf("%v < No data", c)
 		}
 	} else {
-		vrps, exists := e.sdManager.GetCurrentSDs()
+		data, exists := e.sdManager.GetCurrentSDs()
 		if !exists {
 			c.SendInternalError()
 			if e.Log != nil {
 				e.Log.Debugf("%v < Internal error requesting cache (does not exists)", c)
 			}
 		} else {
-			c.SendSDs(sessionId, serial, vrps)
+			c.SendSDs(sessionId, serial, data)
 			if e.Log != nil {
-				e.Log.Debugf("%v < Sent VRPs (current serial %d, session: %d)", c, serial, sessionId)
+				e.Log.Debugf("%v < Sent cache (current serial %d, session: %d)", c, serial, sessionId)
 			}
 		}
 	}
@@ -92,7 +89,7 @@ func (e *DefaultRTREventHandler) RequestNewVersion(c *Client, sessionId uint16,
 	if e.Log != nil {
 		e.Log.Debugf("%v > Request New Version", c)
 	}
-	serverSessionId := e.sdManager.GetSessionId()
+	serverSessionId := e.sdManager.GetSessionId(c.GetVersion())
 	if sessionId != serverSessionId {
 		c.SendCorruptData()
 		if e.Log != nil {
@@ -101,23 +98,23 @@ func (e *DefaultRTREventHandler) RequestNewVersion(c *Client, sessionId uint16,
 		c.Disconnect()
 		return
 	}
-	serial, valid := e.sdManager.GetCurrentSerial(sessionId)
+	serial, valid := e.sdManager.GetCurrentSerial()
 	if !valid {
 		c.SendNoDataError()
 		if e.Log != nil {
 			e.Log.Debugf("%v < No data", c)
 		}
 	} else {
-		vrps, exists := e.sdManager.GetSDsSerialDiff(serialNumber)
+		data, exists := e.sdManager.GetSDsSerialDiff(serialNumber)
 		if !exists {
 			c.SendCacheReset()
 			if e.Log != nil {
 				e.Log.Debugf("%v < Sent cache reset", c)
 			}
 		} else {
-			c.SendSDs(sessionId, serial, vrps)
+			c.SendSDs(sessionId, serial, data)
 			if e.Log != nil {
-				e.Log.Debugf("%v < Sent VRPs (current serial %d, session from client: %d)", c, serial, sessionId)
+				e.Log.Debugf("%v < Sent cache (current serial %d, session from client: %d)", c, serial, sessionId)
 			}
 		}
 	}
@@ -127,7 +124,7 @@ type Server struct {
 	baseVersion uint8
 	clientlock  *sync.RWMutex
 	clients     []*Client
-	sessId      uint16
+	sessId      []uint16
 	connected   int
 	maxconn     int
 
@@ -136,15 +133,14 @@ type Server struct {
 	handler        RTRServerEventHandler
 	simpleHandler  RTREventHandler
 	enforceVersion bool
+	disableBGPSec  bool
+	enableNODELAY  bool
 
 	sdlock          *sync.RWMutex
 	sdListDiff      [][]SendableData
-	sdMapSerial     map[uint32]int
-	sdListSerial    []uint32
 	sdCurrent       []SendableData
 	sdCurrentSerial uint32
 	keepDiff        int
-	manualserial    bool
 
 	pduRefreshInterval uint32
 	pduRetryInterval   uint32
@@ -162,6 +158,9 @@ type ServerConfiguration struct {
 
 	SessId int
 
+	DisableBGPSec	bool
+	EnableNODELAY	bool
+
 	RefreshInterval uint32
 	RetryInterval   uint32
 	ExpireInterval  uint32
@@ -171,7 +170,11 @@ type ServerConfiguration struct {
 }
 
 func NewServer(configuration ServerConfiguration, handler RTRServerEventHandler, simpleHandler RTREventHandler) *Server {
-	sessid := GenerateSessionId()
+	sessids := make([]uint16, 0, int(configuration.ProtocolVersion) + 1)
+	s := GenerateSessionId()
+	for i := 0; i <= int(configuration.ProtocolVersion); i++ {
+		sessids = append(sessids, s + uint16(100 * i))
+	}
 
 	refreshInterval := uint32(3600)
 	if configuration.RefreshInterval != 0 {
@@ -189,38 +192,39 @@ func NewServer(configuration ServerConfiguration, handler RTRServerEventHandler,
 	return &Server{
 		sdlock:       &sync.RWMutex{},
 		sdListDiff:   make([][]SendableData, 0),
-		sdMapSerial:  make(map[uint32]int),
-		sdListSerial: make([]uint32, 0),
 		sdCurrent:    make([]SendableData, 0),
 		keepDiff:     configuration.KeepDifference,
 
 		clientlock:     &sync.RWMutex{},
 		clients:        make([]*Client, 0),
-		sessId:         sessid,
+		sessId:         sessids,
 		maxconn:        configuration.MaxConn,
 		baseVersion:    configuration.ProtocolVersion,
+
 		enforceVersion: configuration.EnforceVersion,
-		handler:        handler,
-		simpleHandler:  simpleHandler,
+		disableBGPSec:	configuration.DisableBGPSec,
 
 		pduRefreshInterval: refreshInterval,
 		pduRetryInterval:   retryInterval,
 		pduExpireInterval:  expireInterval,
 
+		handler:        handler,
+		simpleHandler:  simpleHandler,
+
 		log:        configuration.Log,
 		logverbose: configuration.LogVerbose,
 	}
 }
 
-func ConvertSDListToMap(SDs []SendableData) map[string]SendableData {
-	sdMap := make(map[string]SendableData, len(SDs))
+func ConvertSDListToMap(SDs []SendableData) map[string]uint8 {
+	sdMap := make(map[string]uint8, len(SDs))
 	for _, v := range SDs {
-		sdMap[v.HashKey()] = v
+		sdMap[v.HashKey()] = v.GetFlag()
 	}
 	return sdMap
 }
 
-func ComputeDiff(newSDs, prevSDs []SendableData) (added, removed, unchanged []SendableData) {
+func ComputeDiff(newSDs, prevSDs []SendableData, populateUnchanged bool) (added, removed, unchanged []SendableData) {
 	added = make([]SendableData, 0)
 	removed = make([]SendableData, 0)
 	unchanged = make([]SendableData, 0)
@@ -228,22 +232,22 @@ func ComputeDiff(newSDs, prevSDs []SendableData) (added, removed, unchanged []Se
 	newSDsMap := ConvertSDListToMap(newSDs)
 	prevSDsMap := ConvertSDListToMap(prevSDs)
 
-	for _, vrp := range newSDs {
-		_, exists := prevSDsMap[vrp.HashKey()]
+	for _, item := range newSDs {
+		_, exists := prevSDsMap[item.HashKey()]
 		if !exists {
-			rcopy := vrp.Copy()
+			rcopy := item.Copy()
 			rcopy.SetFlag(FLAG_ADDED)
 			added = append(added, rcopy)
 		}
 	}
-	for _, vrp := range prevSDs {
-		_, exists := newSDsMap[vrp.HashKey()]
+	for _, item := range prevSDs {
+		_, exists := newSDsMap[item.HashKey()]
 		if !exists {
-			rcopy := vrp.Copy()
+			rcopy := item.Copy()
 			rcopy.SetFlag(FLAG_REMOVED)
 			removed = append(removed, rcopy)
-		} else {
-			rcopy := vrp.Copy()
+		} else if populateUnchanged {
+			rcopy := item.Copy()
 			unchanged = append(unchanged, rcopy)
 		}
 	}
@@ -256,25 +260,25 @@ func ApplyDiff(diff, prevSDs []SendableData) []SendableData {
 	diffMap := ConvertSDListToMap(diff)
 	prevSDsMap := ConvertSDListToMap(prevSDs)
 
-	for _, vrp := range prevSDs {
-		_, exists := diffMap[vrp.HashKey()]
+	for _, item := range prevSDs {
+		_, exists := diffMap[item.HashKey()]
 		if !exists {
-			rcopy := vrp.Copy()
+			rcopy := item.Copy()
 			newSDs = append(newSDs, rcopy)
 		}
 	}
-	for _, vrp := range diff {
-		if vrp.GetFlag() == FLAG_ADDED {
-			rcopy := vrp.Copy()
+	for _, item := range diff {
+		if item.GetFlag() == FLAG_ADDED {
+			rcopy := item.Copy()
 			newSDs = append(newSDs, rcopy)
-		} else if vrp.GetFlag() == FLAG_REMOVED {
-			cvrp, exists := prevSDsMap[vrp.HashKey()]
+		} else if item.GetFlag() == FLAG_REMOVED {
+			citem, exists := prevSDsMap[item.HashKey()]
 			if !exists {
-				rcopy := vrp.Copy()
+				rcopy := item.Copy()
 				newSDs = append(newSDs, rcopy)
 			} else {
-				if cvrp.GetFlag() == FLAG_REMOVED {
-					rcopy := vrp.Copy()
+				if citem == FLAG_REMOVED {
+					rcopy := item.Copy()
 					newSDs = append(newSDs, rcopy)
 				}
 			}
@@ -284,42 +288,41 @@ func ApplyDiff(diff, prevSDs []SendableData) []SendableData {
 	return newSDs
 }
 
-func (s *Server) SetManualSerial(v bool) {
-	s.manualserial = v
-}
-
-func (s *Server) GetSessionId() uint16 {
-	return s.sessId
+func (s *Server) GetSessionId(version uint8) uint16 {
+	return s.sessId[version]
 }
 
 func (s *Server) GetCurrentSDs() ([]SendableData, bool) {
 	s.sdlock.RLock()
-	vrp := s.sdCurrent
+	sd := s.sdCurrent
 	s.sdlock.RUnlock()
-	return vrp, true
+	return sd, true
 }
 
 func (s *Server) GetSDsSerialDiff(serial uint32) ([]SendableData, bool) {
 	s.sdlock.RLock()
-	vrp, ok := s.getSDsSerialDiff(serial)
+	sd, ok := s.getSDsSerialDiff(serial)
 	s.sdlock.RUnlock()
-	return vrp, ok
+	return sd, ok
 }
 
 func (s *Server) getSDsSerialDiff(serial uint32) ([]SendableData, bool) {
 	if serial == s.sdCurrentSerial {
 		return []SendableData{}, true
 	}
-
-	vrp := make([]SendableData, 0)
-	index, ok := s.sdMapSerial[serial]
-	if ok {
-		vrp = s.sdListDiff[index]
+	if serial > s.sdCurrentSerial {
+		return nil, false
 	}
-	return vrp, ok
+	diff := int(s.sdCurrentSerial - serial)
+	if diff > len(s.sdListDiff) {
+		return nil, false
+	}
+
+	sd := s.sdListDiff[len(s.sdListDiff) - diff]
+	return sd, true
 }
 
-func (s *Server) GetCurrentSerial(sessId uint16) (uint32, bool) {
+func (s *Server) GetCurrentSerial() (uint32, bool) {
 	s.sdlock.RLock()
 	serial, valid := s.getCurrentSerial()
 	s.sdlock.RUnlock()
@@ -327,112 +330,65 @@ func (s *Server) GetCurrentSerial(sessId uint16) (uint32, bool) {
 }
 
 func (s *Server) getCurrentSerial() (uint32, bool) {
-	return s.sdCurrentSerial, len(s.sdListSerial) > 0
-}
-
-func (s *Server) GenerateSerial() uint32 {
-	s.sdlock.RLock()
-	newserial := s.generateSerial()
-	s.sdlock.RUnlock()
-	return newserial
+	return s.sdCurrentSerial, len(s.sdCurrent) > 0
 }
 
 func (s *Server) generateSerial() uint32 {
 	newserial := s.sdCurrentSerial
-	if !s.manualserial && len(s.sdListSerial) > 0 {
-		newserial = s.sdListSerial[len(s.sdListSerial)-1] + 1
+	if len(s.sdCurrent) > 0 {
+		newserial++
 	}
 	return newserial
 }
 
-func (s *Server) setSerial(serial uint32) {
-	s.sdCurrentSerial = serial
-}
-
-// This function sets the serial. Function must
-// be called before the VRPs data is added.
-func (s *Server) SetSerial(serial uint32) {
-	s.sdlock.RLock()
-	defer s.sdlock.RUnlock()
-	//s.vrpListSerial = make([]uint32, 0)
-	s.setSerial(serial)
-}
-
-func (s *Server) CountVRPs() int {
+func (s *Server) CountSDs() int {
 	s.sdlock.RLock()
 	defer s.sdlock.RUnlock()
 
 	return len(s.sdCurrent)
 }
 
-func (s *Server) AddData(vrps []SendableData) {
+func (s *Server) AddData(new []SendableData) bool {
 	s.sdlock.RLock()
 
-	// a slight hack for now, until we have BGPsec/ASPA support
-	vrpsAsSD := make([]SendableData, 0)
-	for _, v := range vrps {
-		vrpsAsSD = append(vrpsAsSD, v.Copy())
-	}
-
-	added, removed, unchanged := ComputeDiff(vrpsAsSD, s.sdCurrent)
+	added, removed, _ := ComputeDiff(new, s.sdCurrent, false)
 	if s.log != nil && s.logverbose {
-		s.log.Debugf("Computed diff: added (%v), removed (%v), unchanged (%v)", added, removed, unchanged)
+		s.log.Debugf("Computed diff: added (%v), removed (%v)", added, removed)
 	} else if s.log != nil {
-		s.log.Debugf("Computed diff: added (%d), removed (%d), unchanged (%d)", len(added), len(removed), len(unchanged))
+		s.log.Debugf("Computed diff: added (%d), removed (%d)", len(added), len(removed))
 	}
 	curDiff := append(added, removed...)
 	s.sdlock.RUnlock()
 
-	s.AddSDsDiff(curDiff)
-}
-
-func (s *Server) addSerial(serial uint32) []uint32 {
-	removed := make([]uint32, 0)
-	if len(s.sdListSerial) >= s.keepDiff && s.keepDiff > 0 {
-		removeDiff := len(s.sdListSerial) - s.keepDiff
-		removed = s.sdListSerial[0:removeDiff]
-		s.sdListSerial = s.sdListSerial[removeDiff:]
+	if len(curDiff) == 0 {
+		return false
+	} else {
+		s.AddSDsDiff(curDiff)
+		return true
 	}
-	s.sdListSerial = append(s.sdListSerial, serial)
-	return removed
 }
 
 func (s *Server) AddSDsDiff(diff []SendableData) {
 	s.sdlock.RLock()
-	nextDiff := make([][]SendableData, len(s.sdListDiff))
-	for i, prevVrps := range s.sdListDiff {
-		nextDiff[i] = ApplyDiff(diff, prevVrps)
+	nextDiff := make([][]SendableData, len(s.sdListDiff) + 1)
+	for i, prevSDs := range s.sdListDiff {
+		nextDiff[i] = ApplyDiff(diff, prevSDs)
 	}
-	newVrpCurrent := ApplyDiff(diff, s.sdCurrent)
-	curserial, _ := s.getCurrentSerial()
+	newSDCurrent := ApplyDiff(diff, s.sdCurrent)
 	s.sdlock.RUnlock()
 
 	s.sdlock.Lock()
 	defer s.sdlock.Unlock()
 	newserial := s.generateSerial()
-	removed := s.addSerial(newserial)
 
 	nextDiff = append(nextDiff, diff)
-	if len(nextDiff) >= s.keepDiff && s.keepDiff > 0 {
-		nextDiff = nextDiff[len(removed):]
+	if s.keepDiff > 0 && len(nextDiff) > s.keepDiff {
+		nextDiff = nextDiff[len(nextDiff) - s.keepDiff:]
 	}
 
-	s.sdMapSerial[curserial] = len(nextDiff) - 1
-
-	if len(removed) > 0 {
-		for k, v := range s.sdMapSerial {
-			if k != curserial {
-				s.sdMapSerial[k] = v - len(removed)
-			}
-		}
-	}
-
-	for _, removeSerial := range removed {
-		delete(s.sdMapSerial, removeSerial)
-	}
 	s.sdListDiff = nextDiff
-	s.sdCurrent = newVrpCurrent
-	s.setSerial(newserial)
+	s.sdCurrent = newSDCurrent
+	s.sdCurrentSerial = newserial
 }
 
 func (s *Server) SetBaseVersion(version uint8) {
@@ -463,10 +419,6 @@ func (s *Server) GetMaxConnections() int {
 	return s.maxconn
 }
 
-func (s *Server) SetSessionId(sessId uint16) {
-	s.sessId = sessId
-}
-
 func (s *Server) ClientConnected(c *Client) {
 	s.clientlock.Lock()
 	s.clients = append(s.clients, c)
@@ -534,22 +486,23 @@ func (s *Server) Start(bind string) error {
 	return s.loopTCP(tcplist, "tcp", s.acceptClientTCP)
 }
 
-var DisableBGPSec = flag.Bool("disable.bgpsec", false, "Disable sending out BGPSEC Router Keys")
-var DisableASPA = flag.Bool("disable.aspa", false, "Disable sending out ASPA objects")
-
 func (s *Server) acceptClientTCP(tcpconn net.Conn) error {
+	if !s.enableNODELAY {
+		tc, ok := tcpconn.(*net.TCPConn)
+		if ok {
+			tc.SetNoDelay(false)
+		}
+	}
+
 	client := ClientFromConn(tcpconn, s, s)
 	client.log = s.log
 	if s.enforceVersion {
 		client.SetVersion(s.baseVersion)
 	}
 	client.SetIntervals(s.pduRefreshInterval, s.pduRetryInterval, s.pduExpireInterval)
-	if *DisableBGPSec {
+	if s.disableBGPSec {
 		client.DisableBGPsec()
 	}
-	if *DisableASPA {
-		client.DisableASPA()
-	}
 	go client.Start()
 	return nil
 }
@@ -670,28 +623,16 @@ func (s *Server) StartTLS(bind string, config *tls.Config) error {
 func (s *Server) GetClientList() []*Client {
 	s.clientlock.RLock()
 	list := make([]*Client, len(s.clients))
-	for i, c := range s.clients {
-		list[i] = c
-	}
+	copy(list, s.clients)
 	s.clientlock.RUnlock()
 	return list
 }
 
 func (s *Server) NotifyClientsLatest() {
-	serial, _ := s.GetCurrentSerial(s.sessId)
-	s.NotifyClients(serial)
-}
-
-func (s *Server) NotifyClients(serialNumber uint32) {
+	serial, _ := s.GetCurrentSerial()
 	clients := s.GetClientList()
 	for _, c := range clients {
-		c.Notify(s.sessId, serialNumber)
-	}
-}
-
-func (s *Server) SendPDU(pdu PDU) {
-	for _, client := range s.clients {
-		client.SendPDU(pdu)
+		c.Notify(s.GetSessionId(c.GetVersion()), serial)
 	}
 }
 
@@ -736,7 +677,6 @@ type Client struct {
 	expireInterval  uint32
 
 	dontSendBGPsecKeys bool
-	dontSendASPA       bool
 
 	log Logger
 }
@@ -761,10 +701,6 @@ func (c *Client) DisableBGPsec() {
 	c.dontSendBGPsecKeys = true
 }
 
-func (c *Client) DisableASPA() {
-	c.dontSendASPA = true
-}
-
 func (c *Client) SetIntervals(refreshInterval uint32, retryInterval uint32, expireInterval uint32) {
 	c.refreshInterval = refreshInterval
 	c.retryInterval = retryInterval
@@ -781,7 +717,7 @@ func (c *Client) SetDisableVersionCheck(disableCheck bool) {
 }
 
 func (c *Client) checkVersion(newversion uint8) {
-	if (!c.versionset || newversion == c.version) && (newversion == PROTOCOL_VERSION_2 || newversion == PROTOCOL_VERSION_1 || newversion == PROTOCOL_VERSION_0) {
+	if (!c.versionset || newversion == c.version) && (newversion == PROTOCOL_VERSION_1 || newversion == PROTOCOL_VERSION_0) {
 		c.SetVersion(newversion)
 	} else {
 		if c.log != nil {
@@ -886,9 +822,9 @@ func (c *Client) Notify(sessionId uint16, serialNumber uint32) {
 }
 
 type VRP struct {
-	Prefix net.IPNet
-	MaxLen uint8
+	Prefix netip.Prefix
 	ASN    uint32
+	MaxLen uint8
 	Flags  uint8
 }
 
@@ -910,18 +846,13 @@ func (r1 *VRP) Equals(r2 SendableData) bool {
 	}
 
 	r2True := r2.(*VRP)
-	return r1.MaxLen == r2True.MaxLen && r1.ASN == r2True.ASN && r1.Prefix.IP.Equal(r2True.Prefix.IP) && bytes.Equal(r1.Prefix.Mask, r2True.Prefix.Mask)
+
+	return r1.MaxLen == r2True.MaxLen && r1.ASN == r2True.ASN && r1.Prefix == r2True.Prefix
 }
 
 func (r1 *VRP) Copy() SendableData {
-	newprefix := net.IPNet{
-		IP:   make([]byte, len(r1.Prefix.IP)),
-		Mask: make([]byte, len(r1.Prefix.Mask)),
-	}
-	copy(newprefix.IP, r1.Prefix.IP)
-	copy(newprefix.Mask, r1.Prefix.Mask)
 	return &VRP{
-		Prefix: newprefix,
+		Prefix: r1.Prefix,
 		ASN:    r1.ASN,
 		MaxLen: r1.MaxLen,
 		Flags:  r1.Flags}
@@ -983,60 +914,13 @@ func (brk *BgpsecKey) GetFlag() uint8 {
 	return brk.Flags
 }
 
-type VAP struct {
-	Flags       uint8
-	AFI         uint8
-	CustomerASN uint32
-	Providers   []uint32
-}
-
-func (vap *VAP) Type() string {
-	return "ASPA"
-}
-
-func (vap *VAP) String() string {
-	return fmt.Sprintf("ASPA AS%v -> AFI %d, Providers: %v", vap.CustomerASN, vap.AFI, vap.Providers)
-}
-
-func (vap *VAP) HashKey() string {
-	return fmt.Sprintf("%v-%x-%v", vap.CustomerASN, vap.AFI, vap.Providers)
-}
-
-func (r1 *VAP) Equals(r2 SendableData) bool {
-	if r1.Type() != r2.Type() {
-		return false
-	}
-
-	r2True := r2.(*VAP)
-	return r1.CustomerASN == r2True.CustomerASN && fmt.Sprint(r1.Providers) == fmt.Sprint(r2True.Providers) /*This could be made faster*/
-}
-
-func (vap *VAP) Copy() SendableData {
-	cop := VAP{
-		CustomerASN: vap.CustomerASN,
-		AFI:         vap.AFI,
-		Flags:       vap.Flags,
-		Providers:   make([]uint32, 0),
-	}
-	cop.Providers = append(cop.Providers, vap.Providers...)
-	return &cop
-}
-
-func (vap *VAP) SetFlag(f uint8) {
-	vap.Flags = f
-}
-
-func (vap *VAP) GetFlag() uint8 {
-	return vap.Flags
-}
-
 func (c *Client) SendSDs(sessionId uint16, serialNumber uint32, data []SendableData) {
 	pduBegin := &PDUCacheResponse{
 		SessionId: sessionId,
 	}
 	c.SendPDU(pduBegin)
-	for _, data := range data {
-		c.SendData(data.Copy())
+	for _, item := range data {
+		c.SendData(item.Copy())
 	}
 	pduEnd := &PDUEndOfData{
 		SessionId:    sessionId,
@@ -1090,7 +974,8 @@ func (c *Client) SendWrongVersionError() {
 func (c *Client) SendData(sd SendableData) {
 	switch t := sd.(type) {
 	case *VRP:
-		if t.Prefix.IP.To4() == nil && t.Prefix.IP.To16() != nil {
+
+		if t.Prefix.Addr().Is6() {
 			pdu := &PDUIPv6Prefix{
 				Flags:  t.Flags,
 				MaxLen: t.MaxLen,
@@ -1098,7 +983,7 @@ func (c *Client) SendData(sd SendableData) {
 				Prefix: t.Prefix,
 			}
 			c.SendPDU(pdu)
-		} else if t.Prefix.IP.To4() != nil {
+		} else if t.Prefix.Addr().Is4() {
 			pdu := &PDUIPv4Prefix{
 				Flags:  t.Flags,
 				MaxLen: t.MaxLen,
@@ -1113,27 +998,13 @@ func (c *Client) SendData(sd SendableData) {
 		}
 
 		pdu := &PDURouterKey{
-			Version:              c.version, // The RouterKey PDU is unchanged from rfc8210 to draft-ietf-sidrops-8210bis-10
+			Version:              c.version,
 			Flags:                t.Flags,
 			SubjectKeyIdentifier: t.Ski,
 			ASN:                  t.ASN,
 			SubjectPublicKeyInfo: t.Pubkey,
 		}
 		c.SendPDU(pdu)
-	case *VAP:
-		if c.version < 2 || c.dontSendASPA {
-			return
-		}
-
-		pdu := &PDUASPA{
-			Version:           c.version,
-			Flags:             t.Flags,
-			AFIFlags:          t.AFI,
-			ProviderASCount:   uint16(len(t.Providers)),
-			CustomerASNumber:  t.CustomerASN,
-			ProviderASNumbers: t.Providers,
-		}
-		c.SendPDU(pdu)
 	}
 }
 
